#  Drakkar-Software trading-backend
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import trading_backend.exchanges as exchanges


class Binance(exchanges.Exchange):
    SPOT_ID = "T9698EB7"
    MARGIN_ID = None
    FUTURE_ID = "uquVg2pc"
    REF_ID = "135007948"
    IS_SPONSORING = True

    @classmethod
    def get_name(cls):
        return 'binance'

    def _get_order_custom_id(self):
        return f"x-{self._get_id()}{self._exchange.connector.client.uuid22()}"

    def get_orders_parameters(self, params=None) -> dict:
        params = super().get_orders_parameters(params)
        params.update({'newClientOrderId': self._get_order_custom_id()})
        return params

    async def _inner_is_valid_account(self) -> (bool, str):
        try:
            details = await self._exchange.connector.client.sapi_get_apireferral_ifnewuser(
                params=self._exchange._get_params({
                    "apiAgentCode": self._get_id()
                })
            )
            if not details.get("rebateWorking", False):
                return False, f"This account has a referral id and it is not {self.REF_ID}, " \
                              f"which is incompatible ({self.REF_ID} or no referral id is required)"
            if not details.get("ifNewUser", False):
                return False, "This account is not new, which is incompatible"
        except AttributeError:
            return False, "Invalid request parameters"
        return True, None



#  Drakkar-Software trading-backend
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import mock
import trading_backend.exchanges as exchanges


async def create_order_mocked_test(exchange: exchanges.Exchange,
                                   exchange_private_post_order_method_name: str,
                                   exchange_request_referral_key: str,
                                   should_contains: bool = True,
                                   symbol: str = "BTC/USDT",
                                   amount: int = 1,
                                   price: int = 1):
    with mock.patch.object(exchange._exchange.connector.client,
                           exchange_private_post_order_method_name,
                           mock.AsyncMock(return_value={})) as post_order_mock:
        # without referral patch
        await exchange._exchange.connector.client.create_limit_buy_order(symbol, amount, price)
        if should_contains:
            assert exchange._get_id() not in post_order_mock.call_args[0][0].get(exchange_request_referral_key, "")
        else:
            assert exchange._get_id() != post_order_mock.call_args[0][0].get(exchange_request_referral_key, "")

        # with referral patch
        await exchange._exchange.connector.client.create_limit_buy_order(symbol, amount, price,
                                                                         params=exchange.get_orders_parameters())
        if should_contains:
            assert exchange._get_id() in post_order_mock.call_args[0][0].get(exchange_request_referral_key, "")
        else:
            assert exchange._get_id() == post_order_mock.call_args[0][0].get(exchange_request_referral_key, "")
